/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.cropcircle;

import au.com.bytecode.opencsv.bean.ColumnPositionMappingStrategy;
import au.com.bytecode.opencsv.bean.CsvToBean;
import java.io.StringReader;
import java.util.List;
import org.openide.util.NbBundle;

/**
 * Processor for Cropcircle files of sensor type ASC 340
 * @author Gerben Feenstra
 */
public class CropcircleASC340FileProcessor extends CropcircleFileProcessor {

    private static final String LOGKEY_TYPE_DISPLAYNAME = NbBundle.getMessage(CropcircleASC340FileProcessor.class, "sensor.cropcircle.fileprocessor.asc340.log.displayname");
    private static final String[] COLUMS = new String[]{"longitude", "latitude", "fixType", "utcTime", "speed", "course", "sensor", "vi1", "vi2", "r1", "r2", "r3", "appRate"};
    private CsvToBean csv;
    private ColumnPositionMappingStrategy strat;

    public CropcircleASC340FileProcessor(CropCircleDataObject toProcess) {
        super(toProcess);
        strat = new ColumnPositionMappingStrategy();
        strat.setType(CropcircleSensorObservation.class);
        strat.setColumnMapping(COLUMS);
        csv = new CsvToBean();
    }

    /**
     * Parses payload to CropcircleSensorObservation
     * @param payload Cropcircle ASC340 sensor line
     * @return CropcircleSensorObservation object
     * @throws CropcircleParseException
     */
    @Override
    public CropcircleSensorObservation parse(String payload) throws CropcircleParseException {
        try {
            List list = csv.parse(strat, new StringReader(payload));
            return (CropcircleSensorObservation) list.get(0);
        } catch (RuntimeException ex) {
            throw new CropcircleParseException(ex.getCause());
        }
    }

    /**
     * Concrete subclasses must override this method to return the displayname of that sensorprocessor to be displayed in the log.
     * For example: "ASC-340 Cropcircle"
     * 
     * Primary use is for logging, for example: [Info] Start processing of ASC-340 Cropcircle sensor file 010710AD.CSV
     * 
     * @return The log display name
     */
    @Override
    protected String getLogDisplayName() {
        return LOGKEY_TYPE_DISPLAYNAME;
    }

    /**
     * Get the header of the Cropcircle file
     * @return a string containing the expected header
     */
    @Override
    public String getHeader() {
        StringBuilder header = new StringBuilder();
        CropcircleASC340FileDescription[] values = CropcircleASC340FileDescription.values();
        for (CropcircleASC340FileDescription desription : values) {
            header.append(desription);
            header.append(",");
        }
        header.deleteCharAt(header.lastIndexOf(","));
        return header.toString();
    }
}
