/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.cropcircle;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorManager;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.SensorObservation;
import nl.cloudfarming.client.model.SensorMetadata;
import nl.cloudfarming.client.sensor.SensorProcessor;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.openide.filesystems.FileObject;
import org.openide.util.NbBundle;

/**
 * Cropcircle File processor Processes Cropcircle sensor files
 *
 * @author Merijn Zengers
 */
public abstract class CropcircleFileProcessor extends SensorProcessor<CropcircleSensorObservation> {

    // Messages
    private static final String LOGKEY_FAILED_TO_READ_FILE = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.logerror.failed_to_read_file");
    private static final String LOGKEY_PROCESSING_SUMMARY_ERRORMSG = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.logerror.nr_of_records_failed");
    private static final String LOGKEY_PROCESSING_START = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.process_start");
    private static final String LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.nr_of_processed");
    // Other variable declarations
    private final ProgressHandle progressHandle;
    private static final AppLogger LOG = AppLogFactory.getLogger(SensorProcessor.class);
    protected CropCircleDataObject dataObject;
    PropertyEditor old;

    public CropcircleFileProcessor(CropCircleDataObject dataObject) {
        assert dataObject != null;
        this.dataObject = dataObject;
        this.progressHandle = ProgressHandleFactory.createHandle(NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.process_description", dataObject.getName()));
        old = PropertyEditorManager.findEditor(Date.class);

    }

    @Override
    public List<SensorObservation> process() {
        //set temporary editor for date
        PropertyEditorManager.registerEditor(Date.class, UtcTimePropertyEditor.class);
        List<SensorObservation> sensorData = new ArrayList<SensorObservation>();

        try {
            int success = 0;
            int progress = 0;
            int fail = 0;

            LOG.info(LOGKEY_PROCESSING_START, getLogDisplayName(), dataObject.getName());
            FileObject fileObject = dataObject.getPrimaryFile();

            try {
                this.progressHandle.start(Long.valueOf(fileObject.asLines().size()).intValue());

                for (String line : fileObject.asLines()) {
                    progressHandle.progress(++progress);
                    if (!line.equals(getHeader())) {
                        try {
                            sensorData.add(parse(line));
                            success++;
                        } catch (Exception ex) {
                            fail++;
                        }
                    }
                }
                LOG.info(LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG, success);
                if (fail > 0) {
                    LOG.warning(LOGKEY_PROCESSING_SUMMARY_ERRORMSG, fail);
                }
            } finally {
                progressHandle.finish();
            }
        } catch (Exception ex) {
            LOG.error(LOGKEY_FAILED_TO_READ_FILE, dataObject.getName());
            LOG.trace(ex);
        } finally {
            //revert back to default date property editor
            PropertyEditorManager.registerEditor(Date.class, old != null ? old.getClass() : null);
        }
        return sensorData;
    }

    /**
     * Method which will create a SensorMetadata object from the preferences:
     * CropcirclePreference.SENSOR_DATA_FILE_FORMAT
     * CropcirclePreference.CROPCIRCLE_SENSOR_HEIGTH
     * CropcirclePreference.NUMBER_OF_CROPCIRCLE_SENSORS
     * CropcirclePreference.CROPCIRCLE_SENSOR_SPREAD
     * CropcirclePreference.SENSOR_TYPE CropcirclePreference.SPECTRAL_CHANNELS
     *
     * @return created SensorMetadata object
     */
    @Override
    protected SensorMetadata buildMetadataFromPreferences() {
        String fileFormat = getFileDateFormat();
        int sensorHeigth = Integer.parseInt(CropcirclePreference.CROPCIRCLE_SENSOR_HEIGTH.getValue());
        int numberOfSensors = Integer.parseInt(CropcirclePreference.NUMBER_OF_CROPCIRCLE_SENSORS.getValue());
        String cropCircleSensorSpread = CropcirclePreference.CROPCIRCLE_SENSOR_SPREAD.getValue();
        String sensorType = CropcirclePreference.SENSOR_TYPE.getValue();
        String spectralChannels = CropcirclePreference.SPECTRAL_CHANNELS.getValue();
        return new SensorMetadata(fileFormat, sensorHeigth, numberOfSensors, cropCircleSensorSpread, sensorType, spectralChannels);
    }

    @Override
    protected String getFileDateFormat() {
        return CropcirclePreference.SENSOR_DATA_FILE_FORMAT.getValue();
    }

    /**
     * Implemented by subtypes of cropcircle
     *
     * @param payload String (Line/record of a sensorfile) to parse
     * @param dataFile DataFile belongingto the sensorfile
     * @return SensorObservation Object
     * @throws CropcircleParseException
     */
    abstract public CropcircleSensorObservation parse(String payload) throws CropcircleParseException;

    abstract public String getHeader();
}
